--[[
	controlData
	{
		type = "if_enabled",
		name = "My Control(embedded checkbox)",
		tooltipIfEnabled = "Checkbox's tooltip text.(if enabled)",
		tooltipIfDisabled = "Checkbox's tooltip text.(if disabled)",
		disabled = function () -- (optional)
			return db.someBooleanSetting
		end,
		getFunc = function ()
			return db.var
		end,
		setFunc = function ( value )
			db.var = value
			doStuff()
		end,
		widget = { ... } -- embedded destination control data
		default = defalts.var -- (optional)
	}
--]]


local widgetVersion = 1
local LAM = LibStub( "LibAddonMenu-2.0" )

if not LAM:RegisterWidget( "if_enabled", widgetVersion ) 
then
	return 
end

local function UpdateTooltipText( control )

	if control.isDisabled
	then
		control.data.tooltipText = ""
	else
		if control.value
		then
			control.data.tooltipText = control.data.tooltipIfDisabled
		else
			control.data.tooltipText = control.data.tooltipIfEnabled
		end

		if MouseIsOver( control ) and not InformationTooltip:IsControlHidden()
		then
			ZO_Options_OnMouseEnter( control )
		end
	end

end

local function UpdateDisabled( control )

	local isDisabled
	if type( control.data.disabled ) == "function"
	then
		isDisabled = control.data.disabled()
	else
		isDisabled = control.data.disabled
	end

	local checkboxColor = ( isDisabled ) and ZO_DEFAULT_DISABLED_COLOR or ZO_NORMAL_TEXT
	control.checkbox:SetColor( checkboxColor:UnpackRGBA() )

	control.isDisabled = isDisabled
	control:UpdateTooltipText()

end

local function UpdateValue( control, forceDefault, value )

	if forceDefault
	then
		value = control.data.default
		control.data.setFunc( value )
	elseif value ~= nil
	then
		control.data.setFunc( value )
		if control.panel.data.registerForRefresh
		then
			CALLBACK_MANAGER:FireCallbacks( "LAM-RefreshPanel", control )
		end
	else
		value = control.data.getFunc()
	end

	local checkboxText = ( value ) and control.checkedText or control.uncheckedText
	control.checkbox:SetText( checkboxText )
	control.value = value
	control:UpdateTooltipText()

end

function LAMCreateControl.if_enabled( parent, controlData, controlName )

	local function if_enabled_disabled()
		local isDisabled

		if type( controlData.disabled ) == "function"
		then
			isDisabled = controlData.disabled()
		else
			isDisabled = controlData.disabled
		end

		return ( isDisabled or not controlData.getFunc() )
	end

	controlData.widget.disabled = if_enabled_disabled

	local widget = LAMCreateControl[controlData.widget.type]( parent, controlData.widget )
	local control = WINDOW_MANAGER:CreateControl( controlName, widget, CT_CONTROL )

	control:SetMouseEnabled( true )
	control:SetHandler( "OnMouseEnter",
		function ( control )
			ZO_Options_OnMouseEnter( control )
			if not control.isDisabled
			then
				control.checkbox:SetColor( ZO_HIGHLIGHT_TEXT:UnpackRGBA() )
			end
		end )
	control:SetHandler( "OnMouseExit", 
		function ( control )
			ZO_Options_OnMouseExit( control )
			if not control.isDisabled
			then
				control.checkbox:SetColor( ZO_NORMAL_TEXT:UnpackRGBA() )
			end
		end )
	control:SetHandler( "OnMouseUp", 
		function ( control )
			if not control.isDisabled
			then
				PlaySound( SOUNDS.DEFAULT_CLICK )
				control.value = ( not control.value )
				control:UpdateValue( false, control.value )
			end
		end )

	control.checkbox = WINDOW_MANAGER:CreateControl( nil, control, CT_LABEL )
	control.checkbox:SetFont( "ZoFontGameBold" )
	control.checkbox:SetColor( ZO_NORMAL_TEXT:UnpackRGBA() )
	control.checkedText = GetString( SI_CHECK_BUTTON_ON ):upper()
	control.uncheckedText = GetString( SI_CHECK_BUTTON_OFF ):upper()

	control.panel = parent.panel or parent
	control.widget = widget
	control.data = controlData

	local isHalfWidget = ( controlData.widget.width == "half" )
	local extraOffset = ( not isHalfWidget ) and 15 or 0
	control:SetDimensions( 60 ,30 )
	control:SetAnchor( LEFT, widget.label, RIGHT, -( 60 + extraOffset ), 0 )
	control.checkbox:SetDimensions( 48 ,26 )
	control.checkbox:SetAnchor( LEFT, widget.label, RIGHT, -( 48 + extraOffset ), 0 )

	control.UpdateTooltipText = UpdateTooltipText

	if control.data.disabled
	then
		control.UpdateDisabled = UpdateDisabled
		control:UpdateDisabled()
	end

	control.UpdateValue = UpdateValue
	control:UpdateValue()

	if control.panel.data.registerForRefresh or control.panel.data.registerForDefaults
	then
		table.insert( control.panel.controlsToRefresh, control )
	end

	return widget

end
