local ADDON_NAME = MultiQuestTracker_GetAddonName()

local BACKGROUND_FADEOUT_DELAY = 1000
local BACKGROUND_FADE_DURATION = 350

-- 背景処理のラッパークラス
local TrackerBackground = {}

function TrackerBackground:New( texture, control )

	local instance = setmetatable( {}, { __index = self } )

	instance.m_Texture = texture
	instance.m_Texture:MapToControl( control )

	instance.m_FadeAnimation = ZO_AlphaAnimation:New( control )

	return instance

end

function TrackerBackground:IsFading()

	return self.m_FadeAnimation:IsPlaying()

end

function TrackerBackground:FadeOut()

	if self.alphaUnfocused < self.alphaFocused
	then
		self.m_FadeAnimation:FadeOut( BACKGROUND_FADEOUT_DELAY, BACKGROUND_FADE_DURATION )
	end

end

function TrackerBackground:FadeIn()

	if self.alphaFocused > self.alphaUnfocused
	then
		self.m_FadeAnimation:FadeIn( 0, BACKGROUND_FADE_DURATION )
	end

end

function TrackerBackground:GetControl()

	return self.m_Texture:GetMappingControl()

end

function TrackerBackground:SetAlpha( alpha )

	local opacity = alpha.unfocused / 100

	local alphaMax = self.m_Texture:AlphaPercentToValue( alpha.focused )
	local alphaMin = alphaMax * opacity

	self.m_FadeAnimation:SetMinMaxAlpha( alphaMin, alphaMax )

	self:GetControl():SetAlpha( alphaMin )

	self.alphaFocused = alphaMax
	self.alphaUnfocused = alphaMin

end

function TrackerBackground:AdjustTextureSize()

	self.m_Texture:AdjustTextureSize()

end

-- gamepadモードであっても表示スタイルをkeyboard用に固定するためのコードを挿入
local LINE_SPACE_TRACKER_TREE_HEADER = 1
local LINE_SPACE_TRACKER_TREE_CONDITION = 2
local LINE_SPACE_TRACKER_TREE_SUBCATEGORY_TITLE = 3
local LINE_SPACE_TRACKER_TREE_SUBCATEGORY_CONDITION = 4

local TrackerBaseAppearances = 
{
	ANCHOR_PARAMS =
	{
		CONTAINER = { point = TOPLEFT, relPoint = BOTTOMLEFT, X = 35, Y = 5 },
		QUEST_TIMER = { point = TOPLEFT, relPoint = BOTTOMLEFT, X = 10, Y = 0 },
		ASSISTED_TEXTURE = { point = TOPRIGHT, relPoint = TOPLEFT, X = 0, Y = -7 },
		TRACKER_TREE = { point = TOPLEFT, relPoint = BOTTOMLEFT, X = 0, Y = 0 }
	},

	FONT =
	{
		header = "ZoFontGameShadow",
		subCategory = "ZoFontGameShadow",
		general = "ZoFontGameShadow"
	},

	HEADER =
	{
		inheritAlpha = true,
		textType = MODIFY_TEXT_TYPE_NONE,
		hideTextures = true 
	},

	QUEST_LINE = 
	{
		baseWidth = 212,
		headerWidth = 222
	},

	TRACKER_TREE =
	{
		indent = 18,
		lineSpacing = { 18, 2, 2, 2 },
		subCategoryVerticalAlignment = TEXT_ALIGN_TOP
	},
}

function TrackerBaseAppearances:New( tracker )

	local instance = setmetatable( {}, { __index = self } )

	local params = self.ANCHOR_PARAMS

	instance.anchors = {}
	instance.anchors.tree = ZO_Anchor:New( params.TRACKER_TREE.point, tracker.trackerControl, params.TRACKER_TREE.point, params.TRACKER_TREE.X, params.TRACKER_TREE.Y )
	instance.anchors.container = ZO_Anchor:New( params.CONTAINER.point, tracker.timerControl, params.CONTAINER.relPoint, params.CONTAINER.X, params.CONTAINER.Y )
	instance.anchors.timer = ZO_Anchor:New( params.QUEST_TIMER.point, tracker.trackerPanel, params.QUEST_TIMER.point, params.QUEST_TIMER.X, params.QUEST_TIMER.Y )

	return instance

end

function TrackerBaseAppearances:ApplyPlatformStyleToHeader( control )

    control:ClearAnchors()

    control:SetDimensions( self.QUEST_LINE.headerWidth, 0 )
    control:SetModifyTextType( self.HEADER.textType )
    control:SetFont( self.FONT.header )

	if control.headerText ~= nil
	then
		control:SetText( control.headerText )
	end

	control:SetInheritAlpha( self.HEADER.inheritAlpha )

	if control.m_TreeNode ~= nil
	then
		control.m_TreeNode:SetOffsetY( self.TRACKER_TREE.lineSpacing[LINE_SPACE_TRACKER_TREE_HEADER] )
	end

end

function TrackerBaseAppearances:ApplyPlatformStyleToCondition( control )

	control:SetDimensions( self.QUEST_LINE.baseWidth, 0 )
	control:SetFont( self.FONT.general )

	if control.m_TreeNode ~= nil
	then
		control.m_TreeNode:SetOffsetY( self.TRACKER_TREE.lineSpacing[control.entryType] )
	end

end

function TrackerBaseAppearances:ApplyPlatformStyleToStepDescription( control )

	control:SetDimensions( self.QUEST_LINE.baseWidth, 0 )
	control:SetFont( self.FONT.subCategory )
	control:SetVerticalAlignment( self.TRACKER_TREE.subCategoryVerticalAlignment )

	if control.m_TreeNode ~= nil
	then
		control.m_TreeNode:SetOffsetY( self.TRACKER_TREE.lineSpacing[LINE_SPACE_TRACKER_TREE_SUBCATEGORY_TITLE] )
	end

end

function TrackerBaseAppearances:ApplyPlatformStyleToAssistedTexture( control, header )

	local params = self.ANCHOR_PARAMS.ASSISTED_TEXTURE

    control:ClearAnchors()
    control:SetAnchor( params.point, header, params.relPoint, params.X, params.Y )
    control:SetInheritAlpha( self.HEADER.inheritAlpha )

end

local SECONDARY_HEANDER_NAME = "MultiQuestTrackerSecondaryHeader"

local TRACKER_STYLE_HEADER = 1
local TRACKER_STYLE_CONDITION = 2

-- クエストトラッカーの本体クラス。ZO_Tracker ベース
MultiQuestTracker = ZO_Tracker:Subclass()
MultiQuestTracker.STYLE_HEADER = TRACKER_STYLE_HEADER
MultiQuestTracker.STYLE_CONDITION = TRACKER_STYLE_CONDITION

function MultiQuestTracker:New( trackerPanel, trackerControl )

	local instance = ZO_Tracker.New( self, trackerPanel, trackerControl )

	-- HUD の表示条件は Vanilla のクエストトラッカーと同等な感じにする
	_G["MULTI_QUEST_TRACKER_FRAGMENT"] = ZO_HUDFadeSceneFragment:New( instance.trackerPanel:GetNamedChild( "Container" ) )

	HUD_SCENE:AddFragment( MULTI_QUEST_TRACKER_FRAGMENT )
	HUD_UI_SCENE:AddFragment( MULTI_QUEST_TRACKER_FRAGMENT )

	EVENT_MANAGER:RegisterForEvent( "MULTI_QUEST_TRACKER_FRAGMENT", EVENT_PLAYER_DEAD, function () instance:UpdateVisibility() end )
	EVENT_MANAGER:RegisterForEvent( "MULTI_QUEST_TRACKER_FRAGMENT", EVENT_PLAYER_ALIVE, function () instance:UpdateVisibility() end )

	-- SecondaryHeader表示のためにheaderPoolのfactory/resetを拡張
	local originalHeaderFactory = instance.headerPool.customFactoryBehavior
	instance.headerPool:SetCustomFactoryBehavior(
		function ( control )
			if originalHeaderFactory ~= nil
			then
				originalHeaderFactory( control )
			end

			local name = control:GetName() .. SECONDARY_HEANDER_NAME
			local headerSecondary = WINDOW_MANAGER:CreateControlFromVirtual( name, control, SECONDARY_HEANDER_NAME )

			headerSecondary.icon1 = headerSecondary:GetNamedChild( "Icon1" )
			headerSecondary.icon2 = headerSecondary:GetNamedChild( "Icon2" )

			headerSecondary:SetHidden( true )
			headerSecondary.icon1:SetHidden( true )
			headerSecondary.icon2:SetHidden( true )

			control.m_HeaderSecondary = headerSecondary
		end )

	local originalHeaderResetter = instance.headerPool.customResetBehavior
	instance.headerPool:SetCustomResetBehavior(
		function ( control )
			local headerSecondary = control.m_HeaderSecondary

			headerSecondary.icon1.tooltipText = ""
			headerSecondary.icon1:SetTexture( "" )
			headerSecondary.icon1:SetHidden( true )

			headerSecondary.icon2.tooltipText = ""
			headerSecondary.icon2:SetTexture( "" )
			headerSecondary.icon2:SetHidden( true )

			headerSecondary:SetText( "" )
			headerSecondary:SetDimensions( 0, 0 )
			headerSecondary:SetHidden( true )

			if headerSecondary.m_TreeNode ~= nil
			then
				instance.treeView:RemoveNode( headerSecondary.m_TreeNode )
				headerSecondary.m_TreeNode = nil
			end

			if originalHeaderResetter ~= nil
			then
				originalHeaderResetter( control )
			end
		end )

	local baseAppearances = TrackerBaseAppearances:New( instance )
	
	instance.baseAppearances = baseAppearances

    instance.headerPool:SetCustomAcquireBehavior(
    	function ( control )
    		baseAppearances:ApplyPlatformStyleToHeader( control )
    	end )
    instance.conditionPool:SetCustomAcquireBehavior(
    	function ( control )
    		baseAppearances:ApplyPlatformStyleToCondition( control )
    	end )
    instance.stepDescriptionPool:SetCustomAcquireBehavior(
    	function ( control )
    		baseAppearances:ApplyPlatformStyleToStepDescription( control )
    	end )

	instance.treeView = ZO_TreeControl:New( baseAppearances.anchors.tree, TrackerBaseAppearances.TRACKER_TREE.indent )

	-- underline付きのassisted icon
	-- こっちの方が見やすくておすすめ
	instance.newAssistedTexture = GetControl( trackerControl, "NewAssisted" )
	instance.newAssistedTexture.icon = GetControl( instance.newAssistedTexture, "Icon" )
	instance.newAssistedTexture.fadePart = GetControl( instance.newAssistedTexture, "FadePart" )

	instance.maxTracked = QUEST_TRACKER.MAX_TRACKED
	instance.trackerVisualStyles = {}
	instance.isTrackerReady = false

	-- QUEST_TRACKER の SetAssisted を拾うため
	QUEST_TRACKER:RegisterCallback( "QuestTrackerAssistStateChanged", 
		function ( unassistedData, assistedData ) 
			instance:OnQuestTrackerAssistStateChanged( unassistedData, assistedData ) 
		end )

	instance:ApplyPlatformStyle()

	FOCUSED_QUEST_TRACKER.trackerPanel:SetHidden( true )

	return instance

end

function MultiQuestTracker:ApplyPlatformStyle()

	-- Reanchor quest timer
	self.timerControl:ClearAnchors()
	self.baseAppearances.anchors.timer:AddToControl( self.timerControl )

	-- Reanchor the container
	self.trackerControl:ClearAnchors()
	self.baseAppearances.anchors.container:AddToControl( self.trackerControl )

	-- Set the correct style for the controls
	local headers = self.headerPool:GetActiveObjects()
	for _, header in pairs( headers )
	do
		self.baseAppearances:ApplyPlatformStyleToHeader( header )
	end

	local conditions = self.conditionPool:GetActiveObjects()
	for _, condition in pairs( conditions )
	do
		self.baseAppearances:ApplyPlatformStyleToCondition( condition )
	end

	local stepDescriptions = self.stepDescriptionPool:GetActiveObjects()
	for _, stepDescription in pairs( stepDescriptions )
	do
		self.baseAppearances:ApplyPlatformStyleToStepDescription( stepDescription )
	end

	-- Anchor the face button
	local data = self.assistedData
	if data ~= nil
	then
		local header = self:GetHeaderForIndex( data.trackType, data.arg1, data.arg2 )
		self.baseAppearances:ApplyPlatformStyleToAssistedTexture( self.assistedTexture, header )
	end

	-- Update the quest tree anchors
	self.treeView:SetIndent( TrackerBaseAppearances.TRACKER_TREE.indent )
	self.treeView:SetRelativePoint( TrackerBaseAppearances.ANCHOR_PARAMS.TRACKER_TREE.relPoint )
	self:UpdateTreeView()

end

-- AddonMenu の背景設定値をクエストラッカーに反映させるための関数
function MultiQuestTracker:SetBackgroundTexture( texture, alpha )

	local backgroundControl = self.trackerControl:GetParent():GetNamedChild( "Background" )
	self.background = TrackerBackground:New( texture, backgroundControl )

	alpha = alpha or { focused = 100, unfocused = 0 }
	self.background:SetAlpha( alpha )

	self.background:AdjustTextureSize()

end

function MultiQuestTracker:SetBackgroundAlpha( alpha )

	if self.background ~= nil
	then
		self.background:SetAlpha( alpha )
	end

end

-- セカンダリヘッダー(Level表示のところ)のサイズを整えて
-- アイコンがきれいに並ぶようにする
local function ArrangementSecondaryHeaderDimensions( activeHeaders )

	local width = 0
	local height = 0

	for _, questHeader in pairs( activeHeaders )
	do
		local control = questHeader.m_HeaderSecondary
		width = zo_max( control:GetTextWidth(), width )
		height = zo_max( control:GetTextHeight(), height )
	end

	for _, questHeader in pairs( activeHeaders )
	do
		questHeader.m_HeaderSecondary:SetDimensions( width, height )
	end

end

-- トラッキングリストの(再)構築
function MultiQuestTracker:InitialTrackingUpdate()

	self.disableAudio = true
	self.isTrackerReady = false

	self:ClearTracker()

	local numTracked = GetNumTracked()

	local trackedByJournalIndex = {}
	for i = 1, numTracked
	do
		local trackType, arg1, arg2 = GetTrackedByIndex( i )
		if trackType == TRACK_TYPE_QUEST
		then
			trackedByJournalIndex[arg1] = { trackType, arg1, ( arg2 ~= 0 ) and arg2 or nil }
		end
	end

	-- ジャーナルと同じ並び順にする
	local i = 1
	local journalQuests = QUEST_JOURNAL_MANAGER:GetQuestListData()
	for _, quest in pairs( journalQuests )
	do
		local tracked = trackedByJournalIndex[quest.questIndex]
		if tracked ~= nil
		then
			local trackType, arg1, arg2 = unpack( tracked )
			if i <= self.maxTracked
			then
				self:BeginTracking( trackType, arg1 )
				if GetTrackedIsAssisted( trackType, arg1, arg2 )
				then
					local header = self:GetHeaderForIndex( trackType, arg1, arg2 )
					self:SetAssisted( header, true )
				end
			else
				-- 超過分の追跡を解除する
				QUEST_TRACKER:SetTracked( arg1, false )
			end
			i = i + 1
		end
	end

	ArrangementSecondaryHeaderDimensions( self.headerPool:GetActiveObjects() )

	self:UpdateTreeView()
	self:UpdateVisibility()

	self.isTrackerReady = true
	self.disableAudio = false

	self:FireCallbacks( "QuestTrackerInitialUpdate" )

end

-- 個々のクエストをトラッキングリストに載せるための処理を行う
function MultiQuestTracker:BeginTracking( trackType, arg1, arg2 )

	if self:IsOnTracker( trackType, arg1, arg2 ) or self:IsFull() or trackType ~= TRACK_TYPE_QUEST
	then
		return 
	end

	local data = ZO_TrackedData:New( trackType, arg1, arg2 )
	data.level = GetJournalQuestLevel( arg1 )
	data.isComplete = GetJournalQuestIsComplete( arg1 )

	table.insert( self.tracked, data )

	local header = self:AddQuest( data )
	if header ~= nil
	then
		data.header = header
	end

	self.assistedQuestCompleted = false

	self:FireCallbacks( "QuestTrackerTrackingStateChanged", self, true, trackType, arg1, arg2 )

end

function MultiQuestTracker:ClearTracker()

	self.treeView:Clear()
	self.headerPool:ReleaseAllObjects()
	self.assistedTexture:SetHidden( true )
	self.newAssistedTexture:SetHidden( true )

	self.tracked = {}   
	self.assistedData = nil
	self:UpdateTreeView() 
	self:UpdateVisibility()

end

function MultiQuestTracker:StopTracking( trackType, arg1, arg2, completed )

	-- クエストの増減があるとリストの再構築を行うので何もしない

end

-- AddonMenu のヘッダー、Objectiveの設定値をクエストラッカーに反映させるための関数
function MultiQuestTracker:ApplyVisualStyle( styleIndex, styleData, preventTrackerReloading )

	self.trackerVisualStyles[styleIndex] = {}

	ZO_DeepTableCopy( styleData, self.trackerVisualStyles[styleIndex] )

	if not preventTrackerReloading
	then
		self:InitialTrackingUpdate()
	end

end

function MultiQuestTracker:GetMaxNumTracked()

	return self.maxTracked

end

function MultiQuestTracker:SetMaxNumTracked( maxTracked )

	self.maxTracked = maxTracked

end

-- HUDの表示条件
function MultiQuestTracker:UpdateVisibility()

	MULTI_QUEST_TRACKER_FRAGMENT:SetHiddenForReason( "Dead", IsUnitDead( "player" ) )
	MULTI_QUEST_TRACKER_FRAGMENT:SetHiddenForReason( "NoTrackedQuests", ( self:GetNumTracked() == 0 ), DEFAULT_HUD_DURATION, DEFAULT_HUD_DURATION )
    MULTI_QUEST_TRACKER_FRAGMENT:SetHiddenForReason( "DisabledBySetting", ( not GetSetting_Bool( SETTING_TYPE_UI, UI_SETTING_SHOW_QUEST_TRACKER ) ) )

end

function MultiQuestTracker:UpdateTreeView()

	if self:GetNumTracked() > 0
	then
		self.treeView:Update( nil, nil, ZO_Anchor:New( self.baseAppearances.anchors.tree ) )
	end

end

-- クエストヘッダーの文言やらの構築部
function MultiQuestTracker:CreateQuestHeader( data, questName, questType, isComplete )

	local questHeader, treeNode = ZO_Tracker.CreateQuestHeader( self, data, questName, questType, isComplete )

	treeNode:SetOffsetY( TrackerBaseAppearances.TRACKER_TREE.lineSpacing[LINE_SPACE_TRACKER_TREE_HEADER] )

	local questHeaderStyle = self.trackerVisualStyles[TRACKER_STYLE_HEADER]
	if questHeaderStyle ~= nil and questHeaderStyle.showSecondary
	then
		local headerSecondary = questHeader.m_HeaderSecondary
		headerSecondary.m_TreeNode = self.treeView:AddChild( nil, headerSecondary )
		treeNode = headerSecondary.m_TreeNode
	end

	return questHeader, treeNode

end

local function GetQuestLevelText( questHeader )

	local level = questHeader.m_Data.level

	local conColor = ZO_ColorDef:New( GetConColor( level ) )
	local levelString = tostring( level )

	return zo_strformat( SI_QUEST_JOURNAL_QUEST_LEVEL, conColor:Colorize( levelString ) )

end

local function InitializeSecondaryQuestHeader( questHeader, style )

	local headerSecondary = questHeader.m_HeaderSecondary

	headerSecondary:SetFont( style.font )
	headerSecondary:SetColor( style.color:UnpackRGBA() )

	if style.iconSize == nil
	then
		style.iconSize = zo_floor( headerSecondary:GetFontHeight() )
	end

	headerSecondary.icon1:SetDimensions( style.iconSize, style.iconSize )
	headerSecondary.icon2:SetDimensions( style.iconSize, style.iconSize )

	local data = questHeader.m_Data

	headerSecondary:SetText( GetQuestLevelText( questHeader ) )

	local instanceDisplayType = GetJournalInstanceDisplayType( data:GetJournalIndex() )
	if instanceDisplayType ~= INSTANCE_DISPLAY_TYPE_NONE
	then
		local journal = QUEST_JOURNAL_KEYBOARD
		local texture = journal:GetIconTexture( instanceDisplayType )
		headerSecondary.icon1.tooltipText = journal:GetTooltipText( instanceDisplayType )
		headerSecondary.icon1:SetTexture( texture )
		headerSecondary.icon1:SetHidden( false )
	end

	local repeatableType = GetJournalQuestRepeatType( data:GetJournalIndex() )
	if repeatableType ~= QUEST_REPEAT_NOT_REPEATABLE
	then
		local text = GetString( SI_QUEST_JOURNAL_REPEATABLE_TEXT )
		local texture = "EsoUI/Art/Journal/journal_Quest_Repeat.dds"
		if headerSecondary.icon1:GetTextureFileName() ~= ""
		then
			headerSecondary.icon2.tooltipText = text
			headerSecondary.icon2:SetTexture( texture )
			headerSecondary.icon2:SetHidden( false )
		else
			headerSecondary.icon1.tooltipText = text
			headerSecondary.icon1:SetTexture( texture )
			headerSecondary.icon1:SetHidden( false )
		end
	end

end

local function GetQuestHeaderColor( questHeader, style )

	local r, g, b, a

	if style.color ~= nil
	then
		r, g, b, a = style.color:UnpackRGBA()
	else
		r, g, b, a = GetConColor( questHeader.m_Data.level )
	end

	return r, g, b, a

end

-- クエストヘッダーの設定部 ZO_Tracker.CreateQuestHeader() で呼ばれる
function MultiQuestTracker:InitializeQuestHeader( questName, questType, questHeader, isComplete )

	ZO_Tracker.InitializeQuestHeader( self, questName, questType, questHeader, isComplete )

	-- Vanilla のデフォルト値を上書きするような感じで処理する
	local questHeaderStyle = self.trackerVisualStyles[TRACKER_STYLE_HEADER]
	if questHeaderStyle ~= nil
	then
		questHeader:SetColor( GetQuestHeaderColor( questHeader, questHeaderStyle ) )
		questHeader:SetFont( questHeaderStyle.font )

		InitializeSecondaryQuestHeader( questHeader, questHeaderStyle.styleSecondary )
		questHeader.m_HeaderSecondary:SetHidden( not questHeaderStyle.showSecondary )
	end

end

-- Quest Objective の文言やらの構築部 前編
function MultiQuestTracker:PopulateStepQuestConditions( questIndex, stepIndex, questHeader, treeNode, desiredVisibility, entryType )

	if questHeader == nil
	then
		ZO_Tracker.PopulateStepQuestConditions( self, questIndex, stepIndex, questHeader, treeNode, desiredVisibility, entryType )
		return 
	end

	-- 処理前と処理後のテーブルのサイズの差を計って index の範囲を特定する
	local stepDescriptions = questHeader.m_StepDescriptionControls
	local childConditions = questHeader.m_ChildConditionControls

	local prevStepDescSize = nil
	local prevChildCondSize = #childConditions

	if stepDescriptions ~= nil
	then
		 prevStepDescSize = #stepDescriptions
	end

	ZO_Tracker.PopulateStepQuestConditions( self, questIndex, stepIndex, questHeader, treeNode, desiredVisibility, entryType )

	local isStepDescModified = false
	local isChildCondModified = ( #childConditions > prevChildCondSize )

	if stepDescriptions ~= nil
	then
		isStepDescModified = ( #stepDescriptions > prevStepDescSize )
	end

	if isChildCondModified or isStepDescModified
	then
		local visibility = ( select( 2, GetJournalQuestStepInfo( questIndex, stepIndex ) ) )
		local isHintVisibility = ( visibility ~= nil and visibility == QUEST_STEP_VISIBILITY_HINT )
		-- Objective Text が HINT だった場合は、印をつける
		-- この処理は、下の PopulateQuestConditions() で必要
		if isStepDescModified
		then
			for i = prevStepDescSize + 1, #stepDescriptions
			do
				local s = stepDescriptions[i]
				s.m_IsHintVisibility = isHintVisibility
				if s.treeNode ~= nil
				then
					s.treeNode:SetOffsetY( TrackerBaseAppearances.TRACKER_TREE.lineSpacing[LINE_SPACE_TRACKER_TREE_SUBCATEGORY_TITLE] )
				end
			end
		end
		if isChildCondModified
		then
			for i = prevChildCondSize + 1, #childConditions, 2
			do
				local c = childConditions[i]
				c.m_IsHintVisibility = isHintVisibility
				if c.m_TreeNode ~= nil
				then
					c.m_TreeNode:SetOffsetY( TrackerBaseAppearances.TRACKER_TREE.lineSpacing[c.entryType] )
				end
			end
		end
	end

end

-- Quest Objective の以下略 後編
function MultiQuestTracker:PopulateQuestConditions( questIndex, questName, stepType, stepTrackerText, isComplete, tracked, questHeader, treeNode )

	ZO_Tracker.PopulateQuestConditions( self, questIndex, questName, stepType, stepTrackerText, isComplete, tracked, questHeader, treeNode )

	local questCondStyle = self.trackerVisualStyles[TRACKER_STYLE_CONDITION]
	if questCondStyle ~= nil and questHeader ~= nil
	then
		local condColor = questCondStyle.color
		local hintColor = questCondStyle.hintColor
		-- Step descriptions
		if questHeader.m_StepDescriptionControls ~= nil
		then
			for _, stepDescription in pairs( questHeader.m_StepDescriptionControls )
			do
				-- HINT だったら色を変える
				local color = ( stepDescription.m_IsHintVisibility ) and hintColor or condColor
				stepDescription.control:SetFont( questCondStyle.font )
				stepDescription.control:SetColor( color:UnpackRGBA() )
				stepDescription.m_IsHintVisibility = nil
			end
		end
		-- Conditions
		local childConditions = questHeader.m_ChildConditionControls
		for i = 1, #childConditions, 2
		do
			-- HINT だったら色を変える
			local cond = childConditions[i]
			local color = ( cond.m_IsHintVisibility ) and hintColor or condColor
			cond:SetFont( questCondStyle.font )
			cond:SetColor( color:UnpackRGBA() )
			cond.m_IsHintVisibility = nil
		end
	end

end

-- conditionが更新されたときに呼ばれる関数
function MultiQuestTracker:RebuildConditions( questIndex, questHeader, questName, stepType, stepTrackerText, isComplete, tracked )

	-- 引数が不完全(headerなどの主要な引数がnil)な状態で呼ばれる事があるので、
	-- そいつを復元してやる必要がある
	if questHeader == nil
	then
		questHeader = self:GetHeaderForIndex( TRACK_TYPE_QUEST, questIndex )
	end

	if questName == nil
	then
		local _
		questName, _, _, stepType, stepTrackerText, isComplete, tracked = GetJournalQuestInfo( questIndex )
	end

	if questHeader == nil or questName == nil
	then
		return 
	end

	-- 古い情報は削除！
	if questHeader.m_ChildConditionControls ~= nil
	then
		self:RemoveAndReleaseConditionsFromHeader( questHeader )
		questHeader.m_ChildConditionControls = {}
	end

	if questHeader.m_StepDescriptionControls ~= nil
	then
		for _, stepDescription in pairs( questHeader.m_StepDescriptionControls )
		do
			self.stepDescriptionPool:ReleaseObject( stepDescription.key )
			self.treeView:RemoveNode( stepDescription.treeNode )
		end
		questHeader.m_StepDescriptionControls = {}
	end

	-- Secondary Headerが表示してある場合は
	-- nodeの挿入場所をずらす
	local treeNode = questHeader.m_TreeNode
	if questHeader.m_HeaderSecondary.m_TreeNode ~= nil
	then
		treeNode = questHeader.m_HeaderSecondary.m_TreeNode
	end

	-- conditionsの再生成
	self:PopulateQuestConditions( questIndex, questName, stepType, stepTrackerText, isComplete, tracked, questHeader, treeNode )

	-- 閉じてる時ってあるの？
	if treeNode:IsExpanded()
	then
		self:UpdateTreeView()	
	end

end

local MOUSE_ENTER = 1

-- ヘッダーテキストにマウスが乗ったときに色が変わるっぽい
function MultiQuestTracker:DoHeaderNameHighlight( label, state )

	local questHeaderStyle = self.trackerVisualStyles[TRACKER_STYLE_HEADER]
	if questHeaderStyle ~= nil
	then
		local r, g, b, a

		if state == MOUSE_ENTER
		then
			r, g, b, a = ZO_HIGHLIGHT_TEXT:UnpackRGBA()
		else
			r, g, b, a = GetQuestHeaderColor( label, questHeaderStyle )
		end

		label:SetColor( r, g, b, a )
	else
		ZO_Tracker.DoHeaderNameHighlight( self, label, state )
	end

end

-- PlayerLevel が上がったときに QuestLevel の色も変える
function MultiQuestTracker:RefreshHeaderConColors()

	local questHeaderStyle = self.trackerVisualStyles[TRACKER_STYLE_HEADER]
	if questHeaderStyle ~= nil
	then
		local assistedHeader = nil
		local assistedData = self.assistedData

		if assistedData ~= nil
		then
			assistedHeader = self:GetHeaderForIndex( assistedData.trackType, assistedData.arg1, assistedData.arg2 )
		end

		local activeHeaders = self.headerPool:GetActiveObjects()
		for _, questHeader in pairs( activeHeaders )
		do
			local r, g, b, a = GetQuestHeaderColor( questHeader, questHeaderStyle )

			questHeader:SetColor( r, g, b, a )
			-- level textも更新する
			questHeader.m_HeaderSecondary:SetText( GetQuestLevelText( questHeader ) )

			if questHeader == assistedHeader
			then
				-- underlineの色も変える
				self.newAssistedTexture:SetColor( r, g, b, a )
				self.newAssistedTexture.fadePart:SetColor( r, g, b, a )
			end
		end
	else
		ZO_Tracker.RefreshHeaderConColors( self )
	end

end

-- クエストの増減があるとリストの再構築を行う
function MultiQuestTracker:OnQuestAdded( questIndex )

	self:InitialTrackingUpdate()

end

function MultiQuestTracker:OnQuestRemoved( questIndex, completed )

	self:InitialTrackingUpdate()

end

-- QUEST_TRACKER の AssistStateChanged を拾って自身の OnQuestAssistStateChanged() へ再送する
function MultiQuestTracker:OnQuestTrackerAssistStateChanged( unassistedData, assistedData )

	local unassisted = nil

	if unassistedData ~= nil
	then
		local header = self:GetHeaderForIndex( unassistedData.trackType, unassistedData.arg1, unassistedData.arg2 )
		unassisted = ( header ~= nil ) and header.m_Data or nil
		self.assistedData = nil
	end

	if assistedData ~= nil
	then
		local header = self:GetHeaderForIndex( assistedData.trackType, assistedData.arg1, assistedData.arg2 )
		self.assistedData = ( header ~= nil ) and header.m_Data or nil
	end

	self:FireCallbacks( "QuestTrackerAssistStateChanged", unassisted, self.assistedData )

end

local function MoveNewAssistedTexture( control, targetControl )

	control:ClearAnchors()
	control:SetAnchor( TOPLEFT, targetControl, BOTTOMLEFT, -5, -4 )
	control:SetAnchor( TOPRIGHT, targetControl, BOTTOMRIGHT, 5 + 4, -4 )

	control.icon:ClearAnchors()
	control.icon:SetAnchor( LEFT, targetControl, RIGHT, 4, -4 )

end

function MultiQuestTracker:OnQuestAssistStateChanged( unassistedData, assistedData, applyPlatformConstants )

	local questHeaderStyle = self.trackerVisualStyles[TRACKER_STYLE_HEADER]
	if questHeaderStyle == nil or not questHeaderStyle.useNewAssistIcon
	then
		ZO_Tracker.OnQuestAssistStateChanged( self, unassistedData, assistedData, applyPlatformConstants )

		if applyPlatformConstants ~= nil
		then
			local header = self:GetHeaderForIndex( assistedData.trackType, assistedData.arg1, assistedData.arg2 )
			self.baseAppearances:ApplyPlatformStyleToAssistedTexture( self.assistedTexture, header )
		end

		return 
	end

	if unassistedData ~= nil
	then
		self.newAssistedTexture:SetHidden( true )
	end

	if assistedData ~= nil
	then
		local header = self:GetHeaderForIndex( assistedData.trackType, assistedData.arg1, assistedData.arg2 )
		if header ~= nil
		then
			MoveNewAssistedTexture( self.newAssistedTexture, header )

			local r, g, b, a = GetQuestHeaderColor( header, questHeaderStyle )

			self.newAssistedTexture:SetColor( r, g, b, a )
			self.newAssistedTexture.fadePart:SetColor( r, g, b, a )

			self.newAssistedTexture:SetHidden( false )
		end
	end

end

function MultiQuestTracker:SetAssisted( header, showArrows )

	if self.isTrackerReady
	then
		-- ヘッダーテキストをクリックした時はここに来る
		local data = header.m_Data
		local qtHeader = ZO_Tracker.GetHeaderForIndex( QUEST_TRACKER, data.trackType, data.arg1, data.arg2 )
		if qtHeader ~= nil
		then
			ZO_Tracker.SetAssisted( QUEST_TRACKER, qtHeader, showArrows )
		end
	else
		ZO_Tracker.SetAssisted( self, header, showArrows )
	end

end

function MultiQuestTracker:SetTrackTypeAssisted( trackType, show, arg1, arg2 )

	-- QUEST_TRACKER に任せるので何もしない

end

local function GetRegionMapZoneIndex()

	local result = nil

	while true
	do
		result = GetCurrentMapZoneIndex()

		if MapZoomOut() ~= SET_MAP_RESULT_MAP_CHANGED
		then
			-- Tutorial ZoneなどのZoomOut不可なエリア
			break
		end

		local mapType = GetMapType()
		-- MAPTYPE_ALLIANCEはTamriel内の各エリア
		-- MAPTYPE_COSMICはColdharbour
		if mapType == MAPTYPE_ALLIANCE or mapType == MAPTYPE_COSMIC
		then
			break
		end
	end

	return result

end

local function ResetMapLocation()

	SetMapToPlayerLocation()
	-- APIの操作をZO_WorldMap*に通知する
	CALLBACK_MANAGER:FireCallbacks( "OnWorldMapChanged", nil )

end

local PresentLocationQuestStat = {}

function PresentLocationQuestStat:New()

	local instance = setmetatable( {}, { __index = self } )

	instance:SumCurrentRegionQuests()

	return instance

end

local PRESENT_LOCATION_QUEST_STAT = nil

local GetQuestRegionMapZoneIndex
do
	-- クエストの場所情報を引き出す
	-- Currentが変わるので用が済んだらResetする必要あり
	local function GetQuestMapZoneIndex( questIndex )

		if not IsValidQuestIndex( questIndex )
		then
			return nil
		end

		local r = SET_MAP_RESULT_FAILED

		local numSteps = GetJournalQuestNumSteps( questIndex )
		for stepIndex = QUEST_MAIN_STEP_INDEX, numSteps
		do
			local numConditions = GetJournalQuestNumConditions( questIndex, stepInex )
			for conditionIndex = 1, numConditions
			do
				if DoesJournalQuestConditionHavePosition( questIndex, stepIndex, conditionIndex )
				then
					r = SetMapToQuestCondition( questIndex, stepIndex, conditionIndex )
					if r ~= SET_MAP_RESULT_FAILED
					then
						break
					end
				end
			end
		end

		if r == SET_MAP_RESULT_FAILED
		then
			r = SetMapToQuestZone( questIndex )
		end

		if r == SET_MAP_RESULT_FAILED
		then
			return nil
		end

		return GetCurrentMapZoneIndex()

	end

	function GetQuestRegionMapZoneIndex( questIndex )

		if GetQuestMapZoneIndex( questIndex ) == nil
		then
			return nil
		end

		return GetRegionMapZoneIndex()

	end
end

do
	--
	-- "Circonians WaypointIt" コンフリクト回避用コードその1
	--
	local Workaround_For_WaypointIt =
	{
		currentWaypointX,
		currentWaypointY,

		Enter = function ( self )
			self.currentWaypointX, self.currentWaypointY = GetMapPlayerWaypoint()
			if self.currentWaypointX ~= 0 and self.currentWaypointY ~= 0
			then
				local obj = _G["WAYPOINTIT"]
				if obj ~= nil and obj.sv ~= nil
				then
					obj:RunWaypointRemoveUpdates( false )
					obj:RunHeadingUpdates( false )
				end
			end
		end,

		Leave = function ( self )
			if self.currentWaypointX ~= 0 and self.currentWaypointY ~= 0
			then
				local obj = _G["WAYPOINTIT"]
				if obj ~= nil and obj.sv ~= nil
				then
					obj:SetWaypoint( self.currentWaypointX, self.currentWaypointY )
					obj:RunWaypointRemoveUpdates( obj.sv["AUTO_REMOVE_WAYPOINT"] )
					obj:RunHeadingUpdates( obj.sv["WAYPOINT_DIRECTIONAL_ARROW"] )
					self.currentWaypointX = nil
					self.currentWaypointY = nil
				end
			end
		end,
	}

	function PresentLocationQuestStat:GetCurrentRegionMapZoneIndex()

		if self.currentRegionMapZoneIndex == nil
		then
			Workaround_For_WaypointIt:Enter()

			self.currentRegionMapZoneIndex = GetRegionMapZoneIndex()

			Workaround_For_WaypointIt:Leave()
		end

		return self.currentRegionMapZoneIndex

	end

	function PresentLocationQuestStat:SumCurrentRegionQuests()

		Workaround_For_WaypointIt:Enter()

		self.currentRegionMapZoneIndex = GetRegionMapZoneIndex()

		local result = 0

		for i = 1, MAX_JOURNAL_QUESTS
		do
			if IsValidQuestIndex( i )
			then
				local region = GetQuestRegionMapZoneIndex( i )
				if region ~= nil and region == self.currentRegionMapZoneIndex
				then
					result = result + 1
				end
			end
		end

		self.currentRegionQuests = result

		ResetMapLocation()

		Workaround_For_WaypointIt:Leave()

	end
end

function PresentLocationQuestStat:GetNumOfCurrentRegionQuests()

	if self.currentRegionQuests == nil or self.currentRegionMapZoneIndex == nil
	then
		self:SumCurrentRegionQuests()
	end

	return self.currentRegionQuests

end

function MultiQuestTracker_NumOfCurrentRegionQuests()

	return PRESENT_LOCATION_QUEST_STAT:GetNumOfCurrentRegionQuests()

end

function MultiQuestTracker_NumOfCurrentMapQuests()

	return #WORLD_MAP_QUESTS_DATA.masterList

end

local function DoAssistNearestTracked()

	if QUEST_TRACKER:GetNumTracked() > 0 and QUEST_TRACKER.assistedData == nil
	then
		QUEST_TRACKER:AssistClosestTracked()
		if QUEST_TRACKER.assistedData == nil
		then
			for i = 1, MAX_JOURNAL_QUESTS
			do
				if IsValidQuestIndex( i ) and GetIsTracked( TRACK_TYPE_QUEST, i )
				then
					local header = QUEST_TRACKER:GetHeaderForIndex( TRACK_TYPE_QUEST, i )
					if header ~= nil
					then
						QUEST_TRACKER:SetAssisted( header, true )
						break
					end
				end
			end
		end
	end

end

-- マップメニュー右のクエストリスト内のクエストをTrackする
function MultiQuestTracker_TrackCurrentMap()

	if MultiQuestTracker_NumOfCurrentMapQuests() == 0
	then
		return 
	end

	local qtIsMute = QUEST_TRACKER.disableAudio

	QUEST_TRACKER.disableAudio = true

	local masterListIndexes = {}
	for k, v in pairs( WORLD_MAP_QUESTS_DATA.masterList )
	do
		masterListIndexes[v.questIndex] = k
	end

	local nTracked = 0
	local maxTracked = MULTI_QUEST_TRACKER:GetMaxNumTracked()
	local journalQuests = QUEST_JOURNAL_MANAGER:GetQuestListData()
	for _, quest in pairs( journalQuests )
	do
		local index = masterListIndexes[quest.questIndex]
		local isTracked = ( index ~= nil and nTracked < maxTracked )
		if isTracked
		then
			nTracked = nTracked + 1
		end
		QUEST_TRACKER:SetTracked( quest.questIndex, isTracked )
	end

	DoAssistNearestTracked()

	QUEST_TRACKER.disableAudio = qtIsMute

	if not MULTI_QUEST_TRACKER.disableAudio and QUEST_TRACKER:GetNumTracked() > 0
	then
		PlaySound( SOUNDS.QUEST_FOCUSED )
	end

	MULTI_QUEST_TRACKER:InitialTrackingUpdate()

	MultiQuestTracker_UpdateQuestJournal()

end

-- 現在のMapに関係するクエストをTrackする
-- 現地のクエストだけではなく、別のMapのクエストでも
-- 目標が現在のMapにある場合はTrackingの対象になる
function MultiQuestTracker_TrackCurrentRegion()

	local qtIsMute = QUEST_TRACKER.disableAudio

	QUEST_TRACKER.disableAudio = true

	local currentRegionMapZoneIndex = PRESENT_LOCATION_QUEST_STAT:GetCurrentRegionMapZoneIndex()

	local nTracked = 0
	local maxTracked = MULTI_QUEST_TRACKER:GetMaxNumTracked()
	local journalQuests = QUEST_JOURNAL_MANAGER:GetQuestListData()
	for _, quest in pairs( journalQuests )
	do
		local region = GetQuestRegionMapZoneIndex( quest.questIndex )
		local isTracked = ( region ~= nil and region == currentRegionMapZoneIndex and nTracked < maxTracked )
		if isTracked
		then
			nTracked = nTracked + 1
		end
		QUEST_TRACKER:SetTracked( quest.questIndex, isTracked )
	end

	DoAssistNearestTracked()

	QUEST_TRACKER.disableAudio = qtIsMute

	if not MULTI_QUEST_TRACKER.disableAudio and QUEST_TRACKER:GetNumTracked() > 0
	then
		PlaySound( SOUNDS.QUEST_FOCUSED )
	end

	ResetMapLocation()

	MULTI_QUEST_TRACKER:InitialTrackingUpdate()

	MultiQuestTracker_UpdateQuestJournal()

end

-- SavedVariables から以前のtrackedを復元する
function MultiQuestTracker_ImportTrackedList( tracked )

	local questIsTracked = {}

	for i = 1, MAX_JOURNAL_QUESTS
	do
		questIsTracked[i] = true
	end

	if next( tracked ) ~= nil
	then
		local questIndexByName = {}
		local journalQuests = QUEST_JOURNAL_MANAGER:GetQuestListData()
		for _, quest in pairs( journalQuests )
		do
			questIndexByName[quest.categoryName] = questIndexByName[quest.categoryName] or {}
			questIndexByName[quest.categoryName][quest.name] = quest.questIndex
		end

		for categoryName, category in pairs( tracked )
		do
			local categoryInUse = false
			for name, isTracked in pairs( category )
			do
				local isvalidQuest = false
				local questCategory = questIndexByName[categoryName]
				if questCategory ~= nil
				then
					local questIndex = questCategory[name]
					if questIndex ~= nil
					then
						questIsTracked[questIndex] = isTracked
						isvalidQuest = true
						categoryInUse = true
					end
				end
				if not isvalidQuest
				then
					-- 無効なので消す
					tracked[categoryName][name] = nil
				end
			end
			if not categoryInUse
			then
				-- 未使用なので消す
				tracked[categoryName] = nil
			end
		end
	end
	
	for i = 1, MAX_JOURNAL_QUESTS
	do
		if IsValidQuestIndex( i )
		then
			QUEST_TRACKER:SetTracked( i, questIsTracked[i] )
		end
	end

	MULTI_QUEST_TRACKER:InitialTrackingUpdate()

end

-- 現在のtrackedを出力用テーブルに吐き出す
-- QuestIndexは動くので、カテゴリ名とクエスト名で管理を行う
function MultiQuestTracker_ExportTrackedList()

	local tracked = {}

	local journalQuests = QUEST_JOURNAL_MANAGER:GetQuestListData()
	for _, quest in pairs( journalQuests )
	do
		tracked[quest.categoryName] = tracked[quest.categoryName] or {}
		tracked[quest.categoryName][quest.name] = GetIsTracked( TRACK_TYPE_QUEST, quest.questIndex )
	end

	return tracked

end

function MultiQuestTracker:SetEventHandler( eventCode, handler )

	self.trackerPanel:UnregisterForEvent( eventCode )
	
	if handler ~= nil
	then
		self.trackerPanel:RegisterForEvent( eventCode, handler )
	end

end

function MultiQuestTracker:UpdateBackgroundDimensions( baseWidth, baseHeight )

	local offsetX, offsetY = select( 5, self.trackerControl:GetAnchor( 0 ) )

	local width = baseWidth
	local height = baseHeight + ( offsetY * 2.00 )

	local firstHeader = self.trackerControl:GetChild( 3 ) -- 1～2番目はassisted icon
	if firstHeader ~= nil
	then
		local questHeaderStyle = self.trackerVisualStyles[TRACKER_STYLE_HEADER]
		if questHeaderStyle ~= nil and questHeaderStyle.useNewAssistIcon
		then
			local rightDiff = self.newAssistedTexture.fadePart:GetRight() - firstHeader:GetRight()
			width = width + rightDiff + ( offsetX * 1.50 )
		else
			width = width + ( offsetX * 1.35 )
		end
	else
		width = 0
		height = 0
	end

	if self.background ~= nil
	then
		self.background:GetControl():SetDimensions( width, height )
		self.background:AdjustTextureSize()
	end

end

local function OnAddOnLoaded( eventCode, addOnName )

	if addOnName == ADDON_NAME
	then
		MultiQuestTracker_AddonInitialize()
		MultiQuestTracker_LoadSettings()

		--
		-- "Circonians WaypointIt" コンフリクト回避用コードその2
		--
		local function DelayCreatePresentLocationQuestStat()
			local eventName = string.format( "%s_For_WaypointIt", ADDON_NAME )
			EVENT_MANAGER:RegisterForUpdate( eventName, 50,
				function ()
					local obj = _G["WAYPOINTIT"]
					if obj ~= nil and obj.sv ~= nil
					then
						EVENT_MANAGER:UnregisterForUpdate( eventName )
						PRESENT_LOCATION_QUEST_STAT = PresentLocationQuestStat:New()
						-- SetEventHandler()を使うと既存のハンドラを上書きしてしまうので
						EVENT_MANAGER:RegisterForEvent( ADDON_NAME, EVENT_PLAYER_ACTIVATED, function () PRESENT_LOCATION_QUEST_STAT:SumCurrentRegionQuests() end )
					end
				end )
		end

		if WaypointIt ~= nil
		then
			DelayCreatePresentLocationQuestStat()
		else
			PRESENT_LOCATION_QUEST_STAT = PresentLocationQuestStat:New()
			-- SetEventHandler()を使うと既存のハンドラを上書きしてしまうので
			EVENT_MANAGER:RegisterForEvent( ADDON_NAME, EVENT_PLAYER_ACTIVATED, function () PRESENT_LOCATION_QUEST_STAT:SumCurrentRegionQuests() end )
		end


		-- セーブ用ハンドラ
		local saveTracked = function ()
			local tracked = MultiQuestTracker_ExportTrackedList()
			MultiQuestTracker_SetSetting( "tracked", {} ) -- clear table
			MultiQuestTracker_SetSetting( "tracked", tracked )
		end

		-- 念のためにゾーンチェンジ前にtrackedを保存しておく
		MULTI_QUEST_TRACKER:SetEventHandler( EVENT_PLAYER_DEACTIVATED, saveTracked )
		-- InitialTrackingUpdate()後も保存する
		MULTI_QUEST_TRACKER:RegisterCallback( "QuestTrackerInitialUpdate", saveTracked )

		-- Load tracked list
		local tracked = MultiQuestTracker_GetSetting( "tracked" )
		MultiQuestTracker_ImportTrackedList( tracked )

		MultiQuestTracker_RegisterAddonMenu()

		MultiQuestTracker_SetupQuestJournalHook()
		MultiQuestTracker_SetQuestJournalHookCallbackHandler( "OnQuestAdded", function () PRESENT_LOCATION_QUEST_STAT:SumCurrentRegionQuests() end )
		MultiQuestTracker_SetQuestJournalHookCallbackHandler( "OnQuestRemoved", function () PRESENT_LOCATION_QUEST_STAT:SumCurrentRegionQuests() end )

		-- ゾーンチェンジするとtrackedがリセットされる問題？の対策
		-- EVENT_QUEST_LIST_UPDATEDでtrackedがリセットされるようだ
		-- trackedの消失を防ぐために、MULTI_QUEST_TRACKER→QUEST_JOURNALの順でイベントを処理する
		local onQuestListUpdated = function ()
			local tracked = MultiQuestTracker_GetSetting( "tracked" )
			MultiQuestTracker_ImportTrackedList( tracked )
			MultiQuestTracker_UpdateQuestJournal()
		end

		MULTI_QUEST_TRACKER:SetEventHandler( EVENT_QUEST_LIST_UPDATED, onQuestListUpdated )

		-- 現在地以外を開いた状態でマップメニューを閉じて再度開くと現在位置がおかしくなる問題を回避
		local onHidden = function ( oldState, newState )
			if newState == SCENE_HIDDEN
			then
				ResetMapLocation()
			end
		end

		WORLD_MAP_SCENE:RegisterCallback( "StateChange", onHidden )
		GAMEPAD_WORLD_MAP_SCENE:RegisterCallback( "StateChange", onHidden )

		-- keyboardモード切り替え後、初回表示時にAssistedのアイコン表示されない時がある問題を回避
		EVENT_MANAGER:RegisterForEvent( ADDON_NAME, EVENT_GAMEPAD_PREFERRED_MODE_CHANGED,
			function () 
				MultiQuestTracker_UpdateQuestJournal()
			end )

		EVENT_MANAGER:UnregisterForEvent( ADDON_NAME, EVENT_ADD_ON_LOADED )
	end

end

-- XML handlers

-- 監視用関数はトラッカーの端で移動(ドラッグ)した際に MouseOver が外れるのを防ぐ
local function MonitorMouseBehavior( control )

	local background = control:GetNamedChild( "Background" )
	local questContaner = control:GetNamedChild( "QuestContainer" )
	local trackerBackground = MULTI_QUEST_TRACKER.background

	control:SetHandler( "OnUpdate", 
		function ( self )
			local isMouseInside = ( MouseIsOver( background ) or MouseIsOver( questContaner ) )
			if not isMouseInside and not self.isMoving and not trackerBackground:IsFading()
			then
				trackerBackground:FadeOut()
				self.isMouseOver = nil
				self:SetHandler( "OnUpdate", nil )
			end
		end )

end

function MultiQuestTracker_OnMouseEnter( control )

	if not control.isMouseOver
	then
		MULTI_QUEST_TRACKER.background:FadeIn()
		control.isMouseOver = true
		MonitorMouseBehavior( control )
	end

end

function MultiQuestTracker_OnMoveStart( control )

	control.isMoving = true

end

function MultiQuestTracker_OnMoveStop( control )

	MultiQuestTracker_SetSetting( "position", { X = control:GetLeft(), Y = control:GetTop() } )
	MultiQuestTracker_HUDPlacement()

	control.isMoving = nil

end

function MultiQuestTracker_OnResizedToFit( control, desiredWidth, desiredHeight )

	local tracker = _G["MULTI_QUEST_TRACKER"]
	if tracker ~= nil
	then
		tracker:UpdateBackgroundDimensions( desiredWidth, desiredHeight )
	end

end

function MultiQuestTracker_OnSecondaryHeaderIconMouseEnter( control )

	if control.tooltipText ~= nil and control.tooltipText ~= ""
	then
		InitializeTooltip( InformationTooltip, control, BOTTOM, 0, 0, TOP )
		SetTooltipText( InformationTooltip, control.tooltipText )
	end

end

function MultiQuestTracker_OnSecondaryHeaderIconMouseExit( control )

	ClearTooltip( InformationTooltip )

end

function MultiQuestTracker_OnTrackQuestCurrentMapZone()

	if SCENE_MANAGER:IsShowingBaseScene()
	then
		if MultiQuestTracker_NumOfCurrentRegionQuests() > 0
		then
			MultiQuestTracker_TrackCurrentRegion()
		else
			ZO_Alert( UI_ALERT_CATEGORY_ERROR, SOUNDS.NEGATIVE_CLICK, SI_MULTI_QUEST_TRACKER_UNTRACKABLE )
		end
	end

end

function MultiQuestTracker_OnInitialize( control )

	local trackerPanel = control
	local trackerContainer = control:GetNamedChild( "Container" )
	local trackerQuestContainer = trackerContainer:GetNamedChild( "QuestContainer" )

	_G["MULTI_QUEST_TRACKER"] = MultiQuestTracker:New( trackerPanel, trackerQuestContainer )

	-- RETICLE_HIDDEN後にマウスカーソルがトラッカーにあった場合
	-- 正しくfadeinされない問題の対策用
	trackerContainer:RegisterForEvent( EVENT_RETICLE_HIDDEN_UPDATE, 
		function ( eventCode, isHidden ) 
			if isHidden
			then
				if MouseIsOver( trackerQuestContainer )
				then
					MultiQuestTracker_OnMouseEnter( trackerContainer )
				end
			end
		end )

	-- トラッカーにマウスカーソルを置いた状態で、ジャーナルを開いて閉じると
	-- 背景のアルファ値が正しく更新されない時がある問題の対策用
	MULTI_QUEST_TRACKER_FRAGMENT:RegisterCallback( "StateChange",
		function ( oldState, newState )
			if newState == SCENE_HIDDEN
			then
				local background = MULTI_QUEST_TRACKER.background
				local currentAlpha = background:GetControl():GetAlpha()
				if currentAlpha ~= background.alphaUnfocused
				then
					background:FadeOut()
				end
			end
		end )

	EVENT_MANAGER:RegisterForEvent( ADDON_NAME, EVENT_ADD_ON_LOADED, OnAddOnLoaded )

end
