-- Settings for MultiQuestTracker
local LAM = LibStub('LibAddonMenu-2.0')
local LMP = LibStub('LibMediaProvider-1.0')

local MIN_TEXT_SIZE = 10
local MAX_TEXT_SIZE = 26

local TRACKER = _G["MULTI_QUEST_TRACKER"]

local SAVE_VARS_MANAGER = nil

local SETTINGS_LOCAL = nil
local SETTINGS_GLOBAL = nil
local SETTINGS = nil

local REINITIALIZE_NOT_NEEDED = MultiQuestTracker_ToVersionNumber( "1.1.10.1" )

local DEFAULTS =
{
	["position"] = { ["X"] = 1200, ["Y"] = 200 },
	["bgAlpha"] = { ["focused"] = 100, ["unfocused"] = 0 },
	["bgType"] = "simple-gradient",
	["headerOverride"] = false,
	["assistIconType"] = "simple-small-icon",
	["headerFont"] =
	{
		["Font"] = { ["use"] = true, ["value"] = "Univers 67" },
		["Style"] = { ["use"] = true, ["value"] = "soft-shadow-thin" },
		["Size"] = { ["use"] = true, ["value"] = 18 },
		["Color"] = { ["use"] = true, ["value"] = { ["r"] = 0.80, ["g"] = 0.50, ["b"] = 0.00, ["a"] = 1.00 } },
	},
	["showSecondaryHeader"] = false,
	["secondaryHeaderStyle"] =
	{
		["TextSize"] = { ["use"] = true, ["value"] = 18 },
		["TextColor"] = { ["use"] = true, ["value"] = { ["r"] = 1.00, ["g"] = 1.00, ["b"] = 1.00, ["a"] = 1.00 } },
	},
	["qTextOverride"] = false,
	["qTextFont"] =
	{
		["Font"] = { ["use"] = true, ["value"] = "Univers 67" },
		["Style"] = { ["use"] = true, ["value"] = "soft-shadow-thin" },
		["Size"] = { ["use"] = true, ["value"] = 18 },
		["Color"] = { ["use"] = true, ["value"] = { ["r"] = 1.00, ["g"] = 1.00, ["b"] = 1.00, ["a"] = 1.00 } },
		["HintColor"] = { ["use"] = true, ["value"] = { ["r"] = 0.53, ["g"] = 0.58, ["b"] = 0.49, ["a"] = 1.00 } },
	},
	["maxTracked"] = QUEST_TRACKER.MAX_TRACKED,
	["tracked"] = {}
}


local function GetOrDefault( value, defaultValue )

	if value ~= nil
	then
		return value
	else
		return defaultValue
	end

end

local APPLY_SET_DEFAULTS_GLOBAL = 1
local APPLY_CHANGE_GLOBAL_TO_LOCAL = 2
local APPLY_CHANGE_LOCAL_TO_GLOBAL = 3

local PREVENT_TRACKER_RELOADING = true

-- セーブ変数の管理者
-- Player設定とAccount-Wide設定をシームレスに切り替えるための機能を提供する
local SaveVariablesManager = 
{
	New = function ( self, params )
		local instance = setmetatable( {}, { __index = self } )

		local saveName = MultiQuestTracker_GetAddonInfo( "saveVariables" )
		local saveVer = MultiQuestTracker_GetAddonInfo( "saveVersion" )

		SETTINGS_LOCAL = ZO_SavedVars:New( saveName, saveVer, nil, DEFAULTS )
		SETTINGS_GLOBAL = ZO_SavedVars:NewAccountWide( saveName, saveVer, nil, {} )

		local accountData = _G[saveName][params.nameSpace][GetDisplayName()]
		instance.localSave = accountData[GetUnitName( "player" )]
		instance.globalSave =  accountData["$AccountWide"]

		instance:Initialize( params )

		return instance
	end,

	Initialize = function ( self, params )
		local isNotGlobalInitialized = ( not self:IsGlobalSaveInitialized() )

		self.globalSave.appearances = GetOrDefault( self.globalSave.appearances, {} )
		self.globalSave.unsharables = GetOrDefault( self.globalSave.unsharables, {} )
		self.globalSave.useGlobalVarsTable = GetOrDefault( self.globalSave.useGlobalVarsTable, false )

		local unsharables = self.globalSave.unsharables

		local function InitializeUnsharableVar( varName, defaultValue )
			unsharables[varName] = GetOrDefault( unsharables[varName], defaultValue )
		end

		InitializeUnsharableVar( "tracked", true )
		InitializeUnsharableVar( "position", true )

		for i, _ in pairs( DEFAULTS )
		do
			InitializeUnsharableVar( i, false )
		end

		self.m_ApplyActionData =
		{
			[APPLY_SET_DEFAULTS_GLOBAL] =
			{
				destination = self.globalSave.appearances,
				returned = SETTINGS_GLOBAL.appearances,
				source = DEFAULTS
			},
			[APPLY_CHANGE_GLOBAL_TO_LOCAL] = 
			{
				destination = {}, -- 更新の必要なし
				returned = SETTINGS_LOCAL,
				source = {} -- 更新の必要なし
			},
			[APPLY_CHANGE_LOCAL_TO_GLOBAL] = 
			{
				destination = self.globalSave.appearances,
				returned = SETTINGS_GLOBAL.appearances,
				source = self.localSave
			},
		}

		if isNotGlobalInitialized
		then
			self:ApplySaveVariables( APPLY_SET_DEFAULTS_GLOBAL )
		end

		-- 新versionで機能追加などを行った場合にreinitializeが必要な場合があるので
		local version = MultiQuestTracker_GetVersion()
		local initializedVer = GetOrDefault( self.localSave.initialized, 0 )
		if initializedVer ~= version
		then
			if initializedVer < REINITIALIZE_NOT_NEEDED
			then
				params.onInitializeOnce( self )
			end
			self.localSave.initialized = version
		end
	end,

	IsCopyableVar = function ( self, varName )
		return ( not self:IsUnsharableVar( varName ) and DEFAULTS[varName] ~= nil )
	end,

	ApplySaveVariables = function ( self, index )
		local data = self.m_ApplyActionData[index]
		if data == nil
		then
			return nil
		end

		local destination = data.destination
		for i, v in pairs( data.source )
		do
			if self:IsCopyableVar( i )
			then
				if type( v ) == "table"
				then
					destination[i] = ZO_DeepTableCopy( v, destination[i] )
				else
					destination[i] = v
				end
			end
		end

		return data.returned
	end,

	IsUnsharableVar = function ( self, varName )
		return ( self.globalSave.unsharables[varName] == true )
	end,

	IsGlobalSaveInitialized = function ( self )
		return ( self.globalSave.appearances ~= nil and self.globalSave.useGlobalVarsTable ~= nil )
	end,

	IsUsingGlobalVarsTable = function ( self )
		return ( self:IsGlobalSaveInitialized() and self.globalSave.useGlobalVarsTable )
	end,
	
	GetSettings = function ( self )
		return ( self:IsUsingGlobalVarsTable() ) and SETTINGS_GLOBAL.appearances or SETTINGS_LOCAL
	end,

	GetSettingsByVarName = function ( self, varName )
		return ( not self:IsUnsharableVar( varName ) ) and self:GetSettings() or SETTINGS_LOCAL
	end,

	UsingGlobalVarsTable = function ( self, use )
		self.globalSave.useGlobalVarsTable = use
		return self:ApplySaveVariables( ( use ) and APPLY_CHANGE_LOCAL_TO_GLOBAL or APPLY_CHANGE_GLOBAL_TO_LOCAL )
	end,

	SetUnsharableVars = function( self, unsharables )
		ZO_ShallowTableCopy( unsharables, self.globalSave.unsharables )
		local globalAppearances = self.globalSave.appearances
		for i, unsharable in pairs( unsharables )
		do
			if not unsharable then
				if type( globalAppearances[i] ) == "table"
				then
					globalAppearances[i] = ZO_DeepTableCopy( self.localSave[i], globalAppearances[i] )
				else
					globalAppearances[i] = self.localSave[i]
				end
			else
				globalAppearances[i] = nil
			end
		end
	end,
}

function MultiQuestTracker_SetSetting( name, value )

	if name == nil or name == ""
	then
		return 
	end

	local settings = SAVE_VARS_MANAGER:GetSettingsByVarName( name )
	if settings == nil or settings[name] == nil
	then
		return 
	end

	local valueType = type( value )
	if valueType == "table"
	then
		if next( value ) ~= nil -- 中身が入っている場合
		then
			ZO_DeepTableCopy( value, settings[name] )
		else -- 空の場合
			settings[name] = {}
		end
	elseif valueType ~= "nil"
	then
		settings[name] = value
	end

end

function MultiQuestTracker_GetSetting( name )

	if name == nil or name == ""
	then
		return nil
	end

	local settings = SAVE_VARS_MANAGER:GetSettingsByVarName( name )
	if settings == nil or settings[name] == nil
	then
		return nil
	end

	return settings[name]

end

-- クエストトラッカーの背景テクスチャー設定群
local BackgroundTextures = {}

-- "basic" スタイル
-- FOCUSED_QUEST_TRACKER の隠された視覚設定を利用
BackgroundTextures["basic"] =
{
	PATH = "EsoUI/Art/Windows/Gamepad/gp_grunge_scalable.dds",
	HEIGHT = 64,
	WIDTH = 512,
	CAPHEIGHT = 30,

	MapToControl = function ( self, control )
		self.parentControl = control
		self.centerTopCoord = self.CAPHEIGHT / self.HEIGHT
		self.centerBottomCoord = ( self.HEIGHT - self.CAPHEIGHT ) / self.HEIGHT

		self.top = control:GetNamedChild( "Top" )
		self.top:SetHidden( false )
		self.top:SetTexture( self.PATH )
		self.top:SetTextureCoords( 0.00, 1.00, 0.00, self.centerTopCoord )

		self.center = control:GetNamedChild( "Center" )
		self.center:SetHidden( false )
		self.center:SetTexture( self.PATH )
		self.center:SetTextureCoords( 0.00, 1.00, self.centerTopCoord, self.centerBottomCoord )

		self.bottom = control:GetNamedChild( "Bottom" )
		self.bottom:SetHidden( false )
		self.bottom:SetTexture( self.PATH )
		self.bottom:SetTextureCoords( 0.00, 1.00, self.centerBottomCoord, 1.00 )
	end,

	GetMappingControl = function ( self )
		return self.parentControl
	end,

	AlphaPercentToValue = function ( self, alphaPercent )
		return ( alphaPercent / 100 ) * 0.80
	end,

	AdjustTextureSize = function ( self )
		local controlWidth, controlHeight = self.parentControl:GetDimensions()
		local leftCoord = ( self.WIDTH - controlWidth ) / self.WIDTH

		self.top:SetDimensions( controlWidth, self.CAPHEIGHT )
		self.top:SetTextureCoords( leftCoord, 1.00, 0.00, self.centerTopCoord )

		self.center:ClearAnchors()
		self.center:SetDimensions( controlWidth, 4 )
		self.center:SetAnchor( TOPRIGHT, self.top, BOTTOMRIGHT )
		self.center:SetAnchor( BOTTOMRIGHT, self.top, BOTTOMRIGHT, 0, controlHeight - ( self.CAPHEIGHT * 2 ) )
		self.center:SetTextureCoords( leftCoord, 1.00, self.centerTopCoord, self.centerBottomCoord )

		self.bottom:SetDimensions( controlWidth, self.CAPHEIGHT )
		self.bottom:SetTextureCoords( leftCoord, 1.00, self.centerBottomCoord, 1.00 )
	end,
}

-- オリジナル版 MultiQuestTracker の背景に似た "simple-gradient" スタイル
BackgroundTextures["simple-gradient"] =
{
	PATH = "EsoUI/Art/Miscellaneous/window_bg_falloff.dds",
	HEIGHT = 64,
	WIDTH = 4,
	CAPHEIGHT = 0,

	MapToControl = function ( self, control )
		self.parentControl = control
		self.top = control:GetNamedChild( "Top" )
		self.top:SetTexture( self.PATH )
		self.top:SetTextureCoords( 0.00, 1.00, 1.00, 0.00 )

		self.center = control:GetNamedChild( "Center" )
		self.center:SetDimensions( 0, 0 )
		self.center:SetHidden( true )

		self.bottom = control:GetNamedChild( "Bottom" )
		self.bottom:SetDimensions( 0, 0 )
		self.bottom:SetHidden( true )
	end,

	GetMappingControl = function ( self )
		return self.parentControl
	end,

	AlphaPercentToValue = function ( self, alphaPercent )
		return ( alphaPercent / 100 ) * 0.75
	end,

	AdjustTextureSize = function ( self )
		self.top:SetDimensions( self.parentControl:GetDimensions() )
	end,
}

local function CreateBackgroundTexture( bgType )

	bgType = bgType or SETTINGS.bgType

	local background = BackgroundTextures[bgType]
	if background == nil
	then
		background = BackgroundTextures["basic"]
	end

	return setmetatable( {}, { __index = background } )

end

-- 背景以外のクエストトラッカー視覚設定群

local function CreateFont( font, size, option )

	return string.format( "%s|%s|%s", font, size, option )

end

-- ヘッダー
local function CreateSecondaryHeaderStyle( font, option )

	local textSize = select( 2, ZoFontGameShadow:GetFontInfo() )
	local textColor = ZO_SELECTED_TEXT
	local style = SETTINGS.secondaryHeaderStyle

	if SETTINGS.headerOverride
	then
		local c = style.TextColor.value
		textColor = ( style.TextColor.use ) and ZO_ColorDef:New( c.r, c.g, c.b, c.a ) or textColor
		textSize = ( style.TextSize.use ) and style.TextSize.value or textSize
	end

	local styleData =
	{
		font = CreateFont( font, textSize, option ),
		color = textColor,
	}

	return styleData

end

local function ApplyHeaderStyle( preventTrackerReloading )

	local color = nil
	local font, size, option = ZoFontGameShadow:GetFontInfo()

	if SETTINGS.headerOverride
	then
		local headerFont = SETTINGS.headerFont
		font = ( headerFont.Font.use ) and LMP:Fetch( "font", headerFont.Font.value ) or font
		size = ( headerFont.Size.use ) and headerFont.Size.value or size
		option = ( headerFont.Style.use ) and headerFont.Style.value or option

		local c = headerFont.Color.value
		color = ( headerFont.Color.use ) and ZO_ColorDef:New( c.r, c.g, c.b, c.a ) or color
	end

	local styleData =
	{
		font = CreateFont( font, size, option ),
		color = color,
		useNewAssistIcon = ( SETTINGS.assistIconType == "underlined" ),
		showSecondary = SETTINGS.showSecondaryHeader,
		styleSecondary = CreateSecondaryHeaderStyle( font, option ),
	}

	TRACKER:ApplyVisualStyle( TRACKER.STYLE_HEADER, styleData, preventTrackerReloading )

end

-- Objective
local function ApplyConditionStyle( preventTrackerReloading )

	local color = ZO_SELECTED_TEXT
	local hintColor = ZO_HINT_TEXT
	local font, size, option = ZoFontGameShadow:GetFontInfo()

	if SETTINGS.qTextOverride
	then
		local qTextFont = SETTINGS.qTextFont
		font = ( qTextFont.Font.use ) and LMP:Fetch( "font", qTextFont.Font.value ) or font
		size = ( qTextFont.Size.use ) and qTextFont.Size.value or size
		option = ( qTextFont.Style.use ) and qTextFont.Style.value or option

		local c = qTextFont.Color.value
		color = ( qTextFont.Color.use ) and ZO_ColorDef:New( c.r, c.g, c.b, c.a ) or color
		local hc = qTextFont.HintColor.value
		hintColor = ( qTextFont.HintColor.use ) and ZO_ColorDef:New( hc.r, hc.g, hc.b, hc.a ) or hintColor
	end

	local styleData =
	{
		font = CreateFont( font, size, option ),
		color = color,
		hintColor = hintColor,
	}

	TRACKER:ApplyVisualStyle( TRACKER.STYLE_CONDITION, styleData, preventTrackerReloading )

end

-- HUDの座標位置設定
-- クエストトラッカーを動かしたときもこいつが呼ばれる
function MultiQuestTracker_HUDPlacement( position )

	if position == nil
	then
		position = MultiQuestTracker_GetSetting( "position" ) or DEFAULTS.position
	end

	local trackerContainer = TRACKER.trackerPanel:GetNamedChild( "Container" )
	trackerContainer:ClearAnchors()
	trackerContainer:SetAnchor( TOPLEFT, GuiRoot, TOPLEFT, position.X, position.Y )

	local questContainer = trackerContainer:GetNamedChild( "QuestContainer" )
	questContainer:ClearAnchors()

	local offsetX, offsetY = 0, 0

	if SETTINGS.assistIconType == "underlined"
	then
		offsetX = offsetX + 25
		offsetY = offsetY + 4
	else
		offsetX = 30
	end
	
	if SETTINGS.bgType == "simple-gradient"
	then
		offsetY = offsetY + 15
	else
		offsetY = offsetY + 23
	end

	questContainer:SetAnchor( TOPLEFT, trackerContainer, TOPLEFT, offsetX, offsetY )

	TRACKER:SetMaxNumTracked( SETTINGS.maxTracked )

end

local ResetKeybindings_Keyboard
local ResetKeybindings_Gamepad
local Keybindings_InjectOnResetToDefaults
do
	local KEYBORD_DEFAULT = 1
	local GAMEPAD_DEFAULT = 2

	local BINDING_KEYBOARD_KEY = { KEY_T , KEY_SHIFT  }
	local BINDING_GAMEPAD_KEY = { KEY_GAMEPAD_DPAD_DOWN_HOLD, KEY_INVALID }

	local BINDING_BASE_ACTION_NAME = "TRACKQUEST_CURRENT_MAPZONE"
	local BINDING_UI_ACTION_NAME = "UI_TRACKQUEST_CURRENT_MAPZONE"

	local function DoCallSecureProtected( funtionName, ... )
	
		if IsProtectedFunction( funtionName )
		then
			return CallSecureProtected( funtionName, ... )
		else
			return _G[funtionName]( ... )
		end
	
	end

	local DefaultKeybindings =
	{
		[BINDING_BASE_ACTION_NAME] =
		{
			Reset = function ( layerIndex, categoryIndex, actionIndex, defaultKeybindType )
				if defaultKeybindType == KEYBORD_DEFAULT
				then
					DoCallSecureProtected( "BindKeyToAction", layerIndex, categoryIndex, actionIndex, 1, BINDING_KEYBOARD_KEY[1], BINDING_KEYBOARD_KEY[2] )
				elseif defaultKeybindType == GAMEPAD_DEFAULT
				then
					DoCallSecureProtected( "BindKeyToAction", layerIndex, categoryIndex, actionIndex, 2, BINDING_GAMEPAD_KEY[1], BINDING_GAMEPAD_KEY[2] )
				end
			end
		},
		[BINDING_UI_ACTION_NAME] =
		{
			Reset = function ( layerIndex, categoryIndex, actionIndex, defaultKeybindType )
				if defaultKeybindType == KEYBORD_DEFAULT
				then
					DoCallSecureProtected( "BindKeyToAction", layerIndex, categoryIndex, actionIndex, 1, BINDING_KEYBOARD_KEY[1], BINDING_KEYBOARD_KEY[2] )
				end
			end
		}
	}

	local function SetDefaultKeybind( actionName, defaultKeybindType )

		local layerIndex, categoryIndex, actionIndex = GetActionIndicesFromName( actionName )
		if layerIndex ~= nil and categoryIndex ~= nil and actionIndex ~= nil
		then
			local defaultKeybindings = DefaultKeybindings[actionName]
			if defaultKeybindings ~= nil
			then
				defaultKeybindings.Reset( layerIndex, categoryIndex, actionIndex, defaultKeybindType )
			end
		end

	end

	function ResetKeybindings_Keyboard()

		SetDefaultKeybind( BINDING_BASE_ACTION_NAME, KEYBORD_DEFAULT )
		SetDefaultKeybind( BINDING_UI_ACTION_NAME, KEYBORD_DEFAULT )

	end

	function ResetKeybindings_Gamepad()

--		SetDefaultKeybind( BINDING_BASE_ACTION_NAME, GAMEPAD_DEFAULT )

	end

	function Keybindings_InjectOnResetToDefaults( first, second )

		local buttonFirst = ESO_Dialogs["KEYBINDINGS_RESET_KEYBOARD_TO_DEFAULTS"]["buttons"][1]
		local orgFunctionFirst = buttonFirst.callback
		buttonFirst.callback = function ( ... )
			orgFunctionFirst( ... )
			first()
		end

		local buttonSecond = ESO_Dialogs["KEYBINDINGS_RESET_GAMEPAD_TO_DEFAULTS"]["buttons"][1]
		local orgFunctionSecond = buttonSecond.callback
		buttonSecond.callback = function ( ... )
			orgFunctionSecond( ... )
			second()
		end

	end
end

-- AddonMenu 設定
-- 色々あります（説明略）
function MultiQuestTracker_RegisterAddonMenu()

	local fontList = LMP:List( "font" )
	local strings = MultiQuestTracker_GetStrings()

	local fontStyles = 
	{
		"normal",
		"outline",
		"thick-outline",
		"shadow",
		"soft-shadow-thick",
		"soft-shadow-thin" 
	}

	local backgroundTypes = 
	{
		"basic",
		"simple-gradient",
	}

	local assistIconStyle =
	{
		"simple-small-icon",
		"underlined"
	}

	local addonName = MultiQuestTracker_GetAddonInfo( "name" )

	local resetDialogName = string.format( "%s_Settings_ResetKeybindings", addonName )
	ESO_Dialogs[resetDialogName] =
	{
		title =
		{
			text = strings.Button_ResetKeybindings.title,
		},
		mainText =
		{
			text = strings.Button_ResetKeybindings.message,
			align = TEXT_ALIGN_CENTER,
		},
		buttons =
		{
			{
				text = SI_OPTIONS_RESET,
				callback = function()
					ResetKeybindings_Keyboard()
					ResetKeybindings_Gamepad()
				end,
			},
			{
				text = SI_DIALOG_CANCEL,
			},
		},
	}

	local panelData =
	{
		type = "panel",
		name = strings.AddOnTitle,
		displayName = strings.Title,
		author = MultiQuestTracker_GetAddonInfo( "author" ),
		version = MultiQuestTracker_GetVersionDisplayFormat(),
		registerForRefresh = true,
		registerForDefaults = true,
	}

	local panel = LAM:RegisterAddonPanel( addonName .. "_Settings", panelData )

	-- 変更の適用はメニューを閉じるときに一括で行う
	GAME_MENU_SCENE:RegisterCallback( "StateChange",
		function ( oldState, newState )
			if newState == SCENE_HIDDEN and panel.isModified
			then
				MultiQuestTracker_HUDPlacement()

				local texture = CreateBackgroundTexture()
				TRACKER:SetBackgroundTexture( texture, SETTINGS.bgAlpha )

				ApplyHeaderStyle( PREVENT_TRACKER_RELOADING )
				ApplyConditionStyle( PREVENT_TRACKER_RELOADING )

				TRACKER:InitialTrackingUpdate()
				MultiQuestTracker_UpdateQuestJournal()

				panel.isModified = nil
			end
		end )

	-- 変更を検知して更新フラグを立てる
	CALLBACK_MANAGER:RegisterCallback( "LAM-RefreshPanel",
		function ( control )
			if control.panel == panel
			then
				control.panel.isModified = true
			end
		end )

	panelData.resetFunc = function ()
		if SAVE_VARS_MANAGER:IsUsingGlobalVarsTable()
		then
			SETTINGS = SAVE_VARS_MANAGER:UsingGlobalVarsTable( false )
			-- ログインプレイヤーの設定値もリセットする
			ZO_DeepTableCopy( DEFAULTS, SETTINGS )
		else
			MultiQuestTracker_SetSetting( "position", DEFAULTS.position )
		end
		SAVE_VARS_MANAGER:SetUnsharableVars( { position = true } )
		panel.isModified = true
	end

	local optionsData =
	{
		{
			type = "header",
			name = strings.QuestTracker
		},
		{
			type = "slider",
			name = strings.Option_MaxTracked.title,
			tooltip = strings.Option_MaxTracked.help,
			min = 1,
			max = QUEST_TRACKER.MAX_TRACKED,
			step = 1,
			getFunc = function ()
				return SETTINGS.maxTracked
			end,
			setFunc = function ( arg )
				SETTINGS.maxTracked = arg
			end,
			default = DEFAULTS.maxTracked
		},
		{
			type = "header",
			name = strings.bgHeader
		},
		{
			type = "dropdown",
			name = strings.Option_BGType.title,
			tooltip = strings.Option_BGType.help,
			choices = backgroundTypes,
			getFunc = function ()
				return SETTINGS.bgType
			end,
			setFunc = function ( arg )
				SETTINGS.bgType = arg
			end,
			default = DEFAULTS.bgType
		},
		{
			type = "slider",
			name = strings.Option_BGAlpha_Focused.title,
			tooltip = strings.Option_BGAlpha_Focused.help,
			min = 0,
			max = 100,
			step = 1,
			getFunc = function ()
				return SETTINGS.bgAlpha.focused
			end,
			setFunc = function ( arg )
				SETTINGS.bgAlpha.focused = arg
			end,
			default = DEFAULTS.bgAlpha.focused
		},
		{
			type = "slider",
			name = strings.Option_BGAlpha_UnFocused.title,
			tooltip = strings.Option_BGAlpha_UnFocused.help,
			min = 0,
			max = 100,
			step = 1,
			getFunc = function ()
				return SETTINGS.bgAlpha.unfocused
			end,
			setFunc = function ( arg )
				SETTINGS.bgAlpha.unfocused = arg
			end,
			default = DEFAULTS.bgAlpha.unfocused
		},
		{
			type = "header",
			name = strings.HeaderHeader,
		},
		{
			type = "dropdown",
			name = strings.Option_AssistIconType.title,
			tooltip = strings.Option_AssistIconType.help,
			choices = assistIconStyle,
			getFunc = function ()
				return SETTINGS.assistIconType
			end,
			setFunc = function ( arg )
				SETTINGS.assistIconType = arg
			end,
			default = DEFAULTS.assistIconType
		},
		{
			type = "checkbox",
			name = strings.Option_SecondaryHeaderShow.title,
			tooltip = strings.Option_SecondaryHeaderShow.help,
			getFunc = function ()
				return SETTINGS.showSecondaryHeader
			end,
			setFunc = function ( arg )
				SETTINGS.showSecondaryHeader = arg
			end,
			default = DEFAULTS.showSecondaryHeader
		},
		{
			type = "checkbox",
			name = strings.Option_HeaderCustom.title,
			tooltip = strings.Option_HeaderCustom.help,
			getFunc = function ()
				return SETTINGS.headerOverride
			end,
			setFunc = function ( arg )
				SETTINGS.headerOverride = arg
			end,
			default = DEFAULTS.headerOverride
		},
		{
			type = "if_enabled",
			tooltipIfEnabled = strings.Option_EnableThisSetting.help,
			tooltipIfDisabled = strings.Option_DisableThisSetting.help,
			disabled = function ()
				return not SETTINGS.headerOverride
			end,
			getFunc = function ()
				return SETTINGS.headerFont.Font.use
			end,
			setFunc = function ( arg )
				SETTINGS.headerFont.Font.use = arg
			end,
			default = DEFAULTS.headerFont.Font.use,
			widget = 
			{
				type = "dropdown",
				name = strings.Option_HeaderFont.title,
				tooltip = strings.Option_HeaderFont.help,
				choices = fontList,
				getFunc = function ()
					return SETTINGS.headerFont.Font.value
				end,
				setFunc = function ( arg )
					SETTINGS.headerFont.Font.value = arg
				end,
				default = DEFAULTS.headerFont.Font.value
			},
		},
		{
			type = "if_enabled",
			tooltipIfEnabled = strings.Option_EnableThisSetting.help,
			tooltipIfDisabled = strings.Option_DisableThisSetting.help,
			disabled = function ()
				return not SETTINGS.headerOverride
			end,
			getFunc = function ()
				return SETTINGS.headerFont.Style.use
			end,
			setFunc = function ( arg )
				SETTINGS.headerFont.Style.use = arg
			end,
			default = DEFAULTS.headerFont.Style.use,
			widget = 
			{
				type = "dropdown",
				name = strings.Option_HeaderStyle.title,
				tooltip = strings.Option_HeaderStyle.help,
				choices = fontStyles,
				getFunc = function ()
					return SETTINGS.headerFont.Style.value
				end,
				setFunc = function ( arg )
					SETTINGS.headerFont.Style.value = arg
				end,
				default = DEFAULTS.headerFont.Style.value
			},
		},
		{
			type = "if_enabled",
			tooltipIfEnabled = strings.Option_EnableThisSetting.help,
			tooltipIfDisabled = strings.Option_DisableThisSetting.help,
			disabled = function ()
				return not SETTINGS.headerOverride
			end,
			getFunc = function ()
				return SETTINGS.headerFont.Size.use
			end,
			setFunc = function ( arg )
				SETTINGS.headerFont.Size.use = arg
			end,
			default = DEFAULTS.headerFont.Size.use,
			widget = 
			{
				type = "slider",
				name = strings.Option_HeaderSize.title,
				tooltip = strings.Option_HeaderSize.help,
				min = MIN_TEXT_SIZE,
				max = MAX_TEXT_SIZE,
				step = 1,
				getFunc = function () 
					return SETTINGS.headerFont.Size.value
				end,
				setFunc = function ( arg ) 
					SETTINGS.headerFont.Size.value = arg
				end,
				default = DEFAULTS.headerFont.Size.value
			},
		},
		{
			type = "if_enabled",
			tooltipIfEnabled = strings.Option_EnableThisSetting.help,
			tooltipIfDisabled = strings.Option_DisableThisSetting.help,
			disabled = function ()
				return not SETTINGS.headerOverride
			end,
			getFunc = function ()
				return SETTINGS.secondaryHeaderStyle.TextSize.use
			end,
			setFunc = function ( arg )
				SETTINGS.secondaryHeaderStyle.TextSize.use = arg
			end,
			default = DEFAULTS.secondaryHeaderStyle.TextSize.use,
			widget = 
			{
				type = "slider",
				name = strings.Option_SecondaryHeaderSize.title,
				tooltip = strings.Option_SecondaryHeaderSize.help,
				min = MIN_TEXT_SIZE,
				max = MAX_TEXT_SIZE,
				step = 1,
				getFunc = function () 
					return SETTINGS.secondaryHeaderStyle.TextSize.value
				end,
				setFunc = function ( arg ) 
					SETTINGS.secondaryHeaderStyle.TextSize.value = arg
				end,
				default = DEFAULTS.secondaryHeaderStyle.TextSize.value
			},
		},
		{
			type = "if_enabled",
			tooltipIfEnabled = strings.Option_EnableThisSetting.help,
			tooltipIfDisabled = strings.Option_DisableThisSetting.help,
			disabled = function ()
				return not SETTINGS.headerOverride
			end,
			getFunc = function ()
				return SETTINGS.headerFont.Color.use
			end,
			setFunc = function ( arg )
				SETTINGS.headerFont.Color.use = arg
			end,
			default = DEFAULTS.headerFont.Color.use,
			widget = 
			{
				type = "colorpicker",
				name = strings.Option_HeaderColor.title,
				tooltip = strings.Option_HeaderColor.help,
				getFunc = function ()
					local color = SETTINGS.headerFont.Color.value
					return color.r, color.g, color.b, color.a
				end,
				setFunc = function ( r, g, b, a )
					local color = SETTINGS.headerFont.Color.value
					color.r = r
					color.g = g
					color.b = b
					color.a = a
				end,
				default = DEFAULTS.headerFont.Color.value
			},
		},
		{
			type = "if_enabled",
			tooltipIfEnabled = strings.Option_EnableThisSetting.help,
			tooltipIfDisabled = strings.Option_DisableThisSetting.help,
			disabled = function ()
				return not SETTINGS.headerOverride
			end,
			getFunc = function ()
				return SETTINGS.secondaryHeaderStyle.TextColor.use
			end,
			setFunc = function ( arg )
				SETTINGS.secondaryHeaderStyle.TextColor.use = arg
			end,
			default = DEFAULTS.secondaryHeaderStyle.TextColor.use,
			widget = 
			{
				type = "colorpicker",
				name = strings.Option_SecondaryHeaderColor.title,
				tooltip = strings.Option_SecondaryHeaderColor.help,
				getFunc = function ()
					local color = SETTINGS.secondaryHeaderStyle.TextColor.value
					return color.r, color.g, color.b, color.a
				end,
				setFunc = function ( r, g, b, a )
					local color = SETTINGS.secondaryHeaderStyle.TextColor.value
					color.r = r
					color.g = g
					color.b = b
					color.a = a
				end,
				default = DEFAULTS.secondaryHeaderStyle.TextColor.value
			},
		},
		{
			type = "header",
			name = strings.QTextHeader,
		},
		{
			type = "checkbox",
			name = strings.Option_QTextCustom.title,
			tooltip = strings.Option_QTextCustom.help,
			getFunc = function ()
				return SETTINGS.qTextOverride
			end,
			setFunc = function ( arg )
				SETTINGS.qTextOverride = arg
			end,
			default = DEFAULTS.qTextOverride
		},
		{
			type = "if_enabled",
			tooltipIfEnabled = strings.Option_EnableThisSetting.help,
			tooltipIfDisabled = strings.Option_DisableThisSetting.help,
			disabled = function ()
				return not SETTINGS.qTextOverride
			end,
			getFunc = function ()
				return SETTINGS.qTextFont.Font.use
			end,
			setFunc = function ( arg )
				SETTINGS.qTextFont.Font.use = arg
			end,
			default = DEFAULTS.qTextFont.Font.use,
			widget = 
			{
				type = "dropdown",
				name = strings.Option_QTextFont.title,
				tooltip = strings.Option_QTextFont.help,
				choices = fontList,
				getFunc = function ()
					return SETTINGS.qTextFont.Font.value
				end,
				setFunc = function ( arg )
					SETTINGS.qTextFont.Font.value = arg
				end,
				default = DEFAULTS.qTextFont.Font.value
			},
		},
		{
			type = "if_enabled",
			tooltipIfEnabled = strings.Option_EnableThisSetting.help,
			tooltipIfDisabled = strings.Option_DisableThisSetting.help,
			disabled = function ()
				return not SETTINGS.qTextOverride
			end,
			getFunc = function ()
				return SETTINGS.qTextFont.Style.use
			end,
			setFunc = function ( arg )
				SETTINGS.qTextFont.Style.use = arg
			end,
			default = DEFAULTS.qTextFont.Style.use,
			widget = 
			{
				type = "dropdown",
				name = strings.Option_QTextStyle.title,
				tooltip = strings.Option_QTextStyle.help,
				choices = fontStyles,
				getFunc = function ()
					return SETTINGS.qTextFont.Style.value
				end,
				setFunc = function ( arg )
					SETTINGS.qTextFont.Style.value = arg
				end,
				default = DEFAULTS.qTextFont.Style.value
			},
		},
		{
			type = "if_enabled",
			tooltipIfEnabled = strings.Option_EnableThisSetting.help,
			tooltipIfDisabled = strings.Option_DisableThisSetting.help,
			disabled = function ()
				return not SETTINGS.qTextOverride
			end,
			getFunc = function ()
				return SETTINGS.qTextFont.Size.use
			end,
			setFunc = function ( arg )
				SETTINGS.qTextFont.Size.use = arg
			end,
			default = DEFAULTS.qTextFont.Size.use,
			widget = 
			{
				type = "slider",
				name = strings.Option_QTextSize.title,
				tooltip = strings.Option_QTextSize.help,
				min = MIN_TEXT_SIZE,
				max = MAX_TEXT_SIZE,
				step = 1,
				getFunc = function () 
					return SETTINGS.qTextFont.Size.value
				end,
				setFunc = function ( arg ) 
					SETTINGS.qTextFont.Size.value = arg
				end,
				default = DEFAULTS.qTextFont.Size.value
			},
		},
		{
			type = "if_enabled",
			tooltipIfEnabled = strings.Option_EnableThisSetting.help,
			tooltipIfDisabled = strings.Option_DisableThisSetting.help,
			disabled = function ()
				return not SETTINGS.qTextOverride
			end,
			getFunc = function ()
				return SETTINGS.qTextFont.Color.use
			end,
			setFunc = function ( arg )
				SETTINGS.qTextFont.Color.use = arg
			end,
			default = DEFAULTS.qTextFont.Color.use,
			widget = 
			{
				type = "colorpicker",
				name = strings.Option_QTextColor.title,
				tooltip = strings.Option_QTextColor.help,
				getFunc = function ()
					local color = SETTINGS.qTextFont.Color.value
					return color.r, color.g, color.b, color.a
				end,
				setFunc = function ( r, g, b, a )
					local color = SETTINGS.qTextFont.Color.value
					color.r = r
					color.g = g
					color.b = b
					color.a = a
				end,
				default = DEFAULTS.qTextFont.Color.value
			},
		},
		{
			type = "if_enabled",
			tooltipIfEnabled = strings.Option_EnableThisSetting.help,
			tooltipIfDisabled = strings.Option_DisableThisSetting.help,
			disabled = function ()
				return not SETTINGS.qTextOverride
			end,
			getFunc = function ()
				return SETTINGS.qTextFont.HintColor.use
			end,
			setFunc = function ( arg )
				SETTINGS.qTextFont.HintColor.use = arg
			end,
			default = DEFAULTS.qTextFont.HintColor.use,
			widget = 
			{
				type = "colorpicker",
				name = strings.Option_QHintTextColor.title,
				tooltip = strings.Option_QHintTextColor.help,
				getFunc = function ()
					local color = SETTINGS.qTextFont.HintColor.value
					return color.r, color.g, color.b, color.a
				end,
				setFunc = function ( r, g, b, a )
					local color = SETTINGS.qTextFont.HintColor.value
					color.r = r
					color.g = g
					color.b = b
					color.a = a
				end,
				default = DEFAULTS.qTextFont.HintColor.value
			},
		},
		{
			type = "header",
			name = strings.AccountWideSettingsHeader,
		},
		{
			type = "checkbox",
			name = strings.Option_SharedAppearances.title,
			tooltip = strings.Option_SharedAppearances.help,
			getFunc = function ()
				return SAVE_VARS_MANAGER:IsUsingGlobalVarsTable()
			end,
			setFunc = function ( arg )
				-- この設定値のリセットはresetFuncで行うので、ここでのdefault値(nil)は無視する
				if arg ~= nil
				then
					SETTINGS = SAVE_VARS_MANAGER:UsingGlobalVarsTable( arg )
				end
			end,
		},
		{
			type = "checkbox_smallfixed",
			name = strings.Option_SharedHudPosition.title,
			tooltip = strings.Option_SharedHudPosition.help,
			disabled = function ()
				return ( not SAVE_VARS_MANAGER:IsUsingGlobalVarsTable() )
			end,
			getFunc = function ()
				return ( not SAVE_VARS_MANAGER:IsUnsharableVar( "position" ) )
			end,
			setFunc = function ( arg )
				-- この設定値のリセットはresetFuncで行うので、ここでのdefault値(nil)は無視する
				if arg ~= nil
				then
					SAVE_VARS_MANAGER:SetUnsharableVars( { position = ( not arg ) } )
				end
			end,
		},
		{
			type = "header",
			name = strings.ResetKeybindingsHeader,
		},
		{
			type = "button",
			name = strings.Button_ResetKeybindings.title,
			tooltip = strings.Button_ResetKeybindings.help,
			warning = strings.Button_ResetKeybindings.warn,
			func = function ()
				ZO_Dialogs_ShowDialog( resetDialogName )
			end,
		},
		{
			type = "description",
			title = " ",
			text = string.format( "%s\n%s", zo_strformat( strings.Option_MoveText, ZO_Keybindings_GetBindingStringFromAction( "TOGGLE_HUD_UI" ) ), strings.OriginalVersionInfo )
		},
	}

	LAM:RegisterOptionControls( addonName .. "_Settings", optionsData )

end

-- 設定を読み込んでクエストラッカーを初期化する
function MultiQuestTracker_LoadSettings()

	local trackerContainer = TRACKER.trackerPanel:GetNamedChild( "Container" )

	DEFAULTS.position.X = trackerContainer:GetLeft()
	DEFAULTS.position.Y = trackerContainer:GetTop()

	Keybindings_InjectOnResetToDefaults( ResetKeybindings_Keyboard, ResetKeybindings_Gamepad )

	local params =
	{
		nameSpace = "Default",
		onInitializeOnce = function ()
			ResetKeybindings_Keyboard()
			ResetKeybindings_Gamepad()
		end,
	}

	SAVE_VARS_MANAGER = SaveVariablesManager:New( params )
	SETTINGS = SAVE_VARS_MANAGER:GetSettings()

	local texture = CreateBackgroundTexture()
	TRACKER:SetBackgroundTexture( texture, SETTINGS.bgAlpha )

	ApplyHeaderStyle( PREVENT_TRACKER_RELOADING )
	ApplyConditionStyle( PREVENT_TRACKER_RELOADING )

	MultiQuestTracker_HUDPlacement()

end
